@@ -167,6 +167,7 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
   PStringArray arr, arr2;
   PString entry;
   PString purl;
+  PString filter;
 
   char prefix [256] = "";
   char hostname [256] = "";
@@ -177,7 +178,6 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
   gchar *firstname = NULL;
   gchar *surname = NULL;
   gchar *tmp = NULL;
-  gchar *filter = NULL;
 
   gboolean sub_scope = FALSE;
   gboolean is_ils = FALSE;
@@ -209,6 +209,8 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
   attrs += "localityname";
   attrs += "ilsa26214430";
   attrs += "ilsa26279966";
+  attrs += "sipaddress";
+  attrs += "sport";
   attrs += "xstatus";
   if (addressbook->call_attribute)
     attrs += addressbook->call_attribute;
@@ -231,8 +233,16 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
   if (!strcmp (prefix, "ils"))
     is_ils = TRUE;
   
-  if (!ldap.Open (hostname, atoi (port))
-      || !ldap.Bind ()) {
+  if (!ldap.Open (hostname, atoi (port))) {
+   
+    nbr = -1;
+    return NULL;
+  }
+    
+  if (is_ils)
+    ldap.SetOption (0x0011, 2);
+
+  if (!ldap.Bind ()) {
     
     nbr = -1;
     return NULL;
@@ -241,19 +251,30 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
   if (is_ils) /* No url in ILS, and no OR either */ {
    
     if (fullname && strcmp (fullname, ""))
-      filter = g_strdup_printf ("(&(cn=%%)(sn=%%%s%%))", fullname);
+      filter = "(&(cn=%)(sn=%" + PString (fullname) + "%))";
     else if (url && strcmp (url, ""))
-      filter = g_strdup_printf ("(&(cn=%%)(mail=%%%s%%))", url);
+      filter = "(&(cn=%)(mail=%" + PString (url) + "%))";
     else
       filter = g_strdup ("(&(cn=%))");
   }  
   else {
-    
-    filter = g_strdup_printf ("(&(|(cn=*%s%s)(givenname=*%s%s)(sn=*%s%s))(mail=*%s%s)(l=*%s%s))", fullname?fullname:"", fullname?"*":"", fullname?fullname:"", fullname?"*":"", fullname?fullname:"", fullname?"*":"", url?url:"", url?"*":"", location?location:"", location?"*":"");
+
+    if (fullname && strcmp (fullname, ""))
+      filter = "(&(|(cn=*" + PString (fullname) + "*)(givenname=*" + PString (fullname) + "*)(sn=*" + PString (fullname) + "*))";
+    else
+      filter = "(&(|(cn=*)(givenname=*)(sn=*))";
+
+    if (url && strcmp (url, ""))
+      filter += "(mail=*" + PString (url) + "*)";
+
+    if (location && strcmp (location, ""))
+      filter += "(l=*" + PString (location) + "*)";
+
+    filter += ")";
   }
 
   if (ldap.Search (context, 
-		   filter, 
+		   (const char *) filter, 
 		   attrs, 
 		   base, 
 		   (sub_scope) 
@@ -334,36 +355,56 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
 
 
       if (ldap.GetSearchResult (context, "ilsa26214430", arr))
-	contact->state = atoi ((const char *) arr [0]);
+	contact->state = (ContactState)atoi ((const char *) arr [0]);
       else if (PString (base).Find ("dc=ekiga") != P_MAX_INDEX) {
 
 	/* Hack for eKiga users. An user is offline if its software
 	 * is NULL.
 	 */
 	if (!contact->software)
-	  contact->state = -1;
+	  contact->state = CONTACT_OFFLINE;
       }
       else
-	contact->state = 0;
+	contact->state = CONTACT_AVAILABLE;
 
-  
-      if (addressbook->call_attribute
-	  && ldap.GetSearchResult (context, addressbook->call_attribute, arr)) {
-	
-	/* Some clever guessing */
-	if (is_ils && !strcasecmp (addressbook->call_attribute, "rfc822Mailbox"))
-	  purl = PString ("callto:") + PString (hostname)
-	    + PString ("/") + PString ((const char *) arr [0]);
-	else {
-	  
-	  purl = PString ("sip:") + PString ((const char *) arr [0]);
-	  purl.Replace ("+", "");
-	  purl.Replace ("-", "");
-	  purl.Replace (" ", "");
-	}
-      
-	contact->url = get_fixed_utf8 ((const char *) purl);
+      purl = PString::Empty();
+      if (is_ils) {
+
+        if (ldap.GetSearchResult (context, "sipaddress", arr)) {
+
+          int part1, part2, part3, part4 = 0;
+          int sport = 1720;
+          gchar *ip = NULL;
+
+          part1 = (atoi (arr [0]) & 0xff000000) >> 24;
+          part2 = (atoi (arr [0]) & 0x00ff0000) >> 16;
+          part3 = (atoi (arr [0]) & 0x0000ff00) >> 8;
+          part4 = atoi (arr [0]) & 0x000000ff;
+
+          if (ldap.GetSearchResult (context, "sport", arr))
+            sport = atoi (arr [0]);
+
+          ip = g_strdup_printf ("h323:%d.%d.%d.%d:%d", 
+                                part4, part3, part2, part1, sport);
+          purl = ip;
+          g_free (ip);
+        }
       }
+      else {
+
+        if (addressbook->call_attribute
+            && ldap.GetSearchResult (context, 
+                                     addressbook->call_attribute, 
+                                     arr)) {
+
+          purl = PString ("sip:") + PString ((const char *) arr [0]);
+          purl.Replace ("+", "");
+          purl.Replace ("-", "");
+          purl.Replace (" ", "");
+        }
+      }
+
+      contact->url = get_fixed_utf8 ((const char *) purl);
       
       list = g_slist_append (list, (gpointer) contact);
 
@@ -375,8 +416,6 @@ gnomemeeting_ldap_addressbook_get_contacts (GmAddressbook *addressbook,
     } while (ldap.GetNextSearchResult (context));
   }
 
-  g_free (filter);
-
   if (nbr != -1) {
    
     if (xstatus != 0)
