@@ -64,6 +64,7 @@
 GMSIPEndpoint::GMSIPEndpoint (GMManager & ep)
 : SIPEndPoint (ep), endpoint (ep)
 {
+  NoAnswerTimer.SetNotifier (PCREATE_NOTIFIER (OnNoAnswerTimeout));
 }
 
 
@@ -115,11 +116,14 @@ BOOL
 GMSIPEndpoint::StartListener (PString iface, 
 			      WORD port)
 {
+  PString iface_noip;
   PString ip;
   PIPSocket::InterfaceTable ifaces;
   PINDEX i = 0;
+  PINDEX pos = 0;
   
   gboolean ok = FALSE;
+  gboolean found = FALSE;
 
   gchar *listen_to = NULL;
 
@@ -132,15 +136,34 @@ GMSIPEndpoint::StartListener (PString iface,
     
     ip = " [" + ifaces [i].GetAddress ().AsString () + "]";
     
-    if (ifaces [i].GetName () + ip == iface)
+    if (ifaces [i].GetName () + ip == iface) {
       listen_to = 
 	g_strdup_printf ("udp$%s:%d", 
 			 (const char *) ifaces [i].GetAddress().AsString(),
 			 port);
+      found = TRUE;
+    }
       
     i++;
   }
 
+  i = 0;
+  pos = iface.Find("[");
+  if (pos != P_MAX_INDEX)
+    iface_noip = iface.Left (pos).Trim ();
+  while (i < ifaces.GetSize() && !found) {
+
+    if (ifaces [i].GetName () == iface_noip) {
+      listen_to = 
+	g_strdup_printf ("udp$%s:%d", 
+			 (const char *) ifaces [i].GetAddress().AsString(),
+			 port);
+      found = TRUE;
+    }
+    
+    i++;
+  }
+
   /* Start the listener thread for incoming calls */
   if (!listen_to)
     return FALSE;
@@ -341,6 +364,7 @@ GMSIPEndpoint::OnIncomingConnection (OpalConnection &connection)
   IncomingCallMode icm;
   gboolean busy_forward = FALSE;
   gboolean always_forward = FALSE;
+  int no_answer_timeout = FALSE;
 
   BOOL res = FALSE;
 
@@ -354,6 +378,8 @@ GMSIPEndpoint::OnIncomingConnection (OpalConnection &connection)
   always_forward = gm_conf_get_bool (CALL_FORWARDING_KEY "always_forward");
   icm =
     (IncomingCallMode) gm_conf_get_int (CALL_OPTIONS_KEY "incoming_call_mode");
+  no_answer_timeout =
+    gm_conf_get_int (CALL_OPTIONS_KEY "no_answer_timeout");
   gnomemeeting_threads_leave ();
 
   call = endpoint.FindCallWithLock (endpoint.GetCurrentCallToken());
@@ -380,6 +406,9 @@ GMSIPEndpoint::OnIncomingConnection (OpalConnection &connection)
   else
     reason = 0; // Ask the user
 
+  if (reason == 0)
+    NoAnswerTimer.SetInterval (0, PMIN (no_answer_timeout, 60));
+
   res = endpoint.OnIncomingConnection (connection, reason, forward_host);
 
   g_free (forward_host);
@@ -446,6 +475,9 @@ GMSIPEndpoint::OnReceivedMESSAGE (OpalTransport & transport,
   PINDEX j = from.Find (';');
   if (j != P_MAX_INDEX)
     from = from.Left(j); // Remove all parameters
+  j = from.Find ('<');
+  if (j != P_MAX_INDEX && from.Find ('>') == P_MAX_INDEX)
+    from += '>';
 
   last = msgData.GetAt (SIPURL (from).AsString ());
   if (!last || *last != pdu.GetMIME ().GetCallID ()) {
@@ -542,20 +574,87 @@ GMSIPEndpoint::GetRegisteredAccounts ()
 
 
 SIPURL
-GMSIPEndpoint::GetDefaultRegisteredPartyName ()
+GMSIPEndpoint::GetRegisteredPartyName (const PString & host)
 {
   GmAccount *account = NULL;
-  
+
   PString url;
+  SIPURL registration_address;
+
+  PSafePtr<SIPInfo> info = activeSIPInfo.FindSIPInfoByDomain(host, SIP_PDU::Method_REGISTER, PSafeReadOnly);
+
+  if (info != NULL)
+    registration_address = info->GetRegistrationAddress();
 
   account = gnomemeeting_get_default_account ("SIP");
-  if (account) {
+  if (account && account->enabled) {
 
-    if (PString(account->username).Find("@") == P_MAX_INDEX)
-      url = PString (account->username) + "@" + PString (account->host);
+    if (info == NULL || registration_address.GetHostName () == account->host) {
 
-    return url;
+      if (PString(account->username).Find("@") == P_MAX_INDEX)
+        url = PString (account->username) + "@" + PString (account->host);
+      else
+        url = PString (account->username);
+
+      return url;
+    }
   }
-  
-  return SIPEndPoint::GetDefaultRegisteredPartyName (); 
+  if (info != NULL) 
+    return registration_address;
+  else 
+    return SIPEndPoint::GetDefaultRegisteredPartyName (); 
 }
+
+
+void 
+GMSIPEndpoint::OnEstablished (OpalConnection &connection)
+{
+  NoAnswerTimer.Stop ();
+
+  PTRACE (3, "GMSIPEndpoint\t SIP connection established");
+  SIPEndPoint::OnEstablished (connection);
+}
+
+
+void 
+GMSIPEndpoint::OnReleased (OpalConnection &connection)
+{
+  NoAnswerTimer.Stop ();
+
+  PTRACE (3, "GMSIPEndpoint\t SIP connection released");
+  SIPEndPoint::OnReleased (connection);
+}
+
+
+void
+GMSIPEndpoint::OnNoAnswerTimeout (PTimer &,
+                                  INT) 
+{
+  gchar *forward_host = NULL;
+  gboolean forward_on_no_answer = FALSE;
+  
+  if (endpoint.GetCallingState () == GMManager::Called) {
+   
+    gnomemeeting_threads_enter ();
+    forward_host = gm_conf_get_string (SIP_KEY "forward_host");
+    forward_on_no_answer = 
+      gm_conf_get_bool (CALL_FORWARDING_KEY "forward_on_no_answer");
+    gnomemeeting_threads_leave ();
+
+    if (forward_host && forward_on_no_answer) {
+      
+      PSafePtr<OpalCall> call = 
+        endpoint.FindCallWithLock (endpoint.GetCurrentCallToken ());
+      PSafePtr<OpalConnection> con = 
+        endpoint.GetConnection (call, TRUE);
+    
+      con->ForwardCall (forward_host);
+    }
+    else
+      ClearAllCalls (OpalConnection::EndedByNoAnswer, FALSE);
+
+    g_free (forward_host);
+  }
+}
+
+
